{ KEYWORDS: VERSION }
{ VERSION: 2.6 }
{ 2.6 / 6.8.2005
  - nova schopnost: TControlEnabler zachovava focus
  2.5 / 5.2.2005
  - nove fce DelayedNotify() funguje jako NotifyWhenIdle(), ale spusti se az
    po uplynuti urcite doby
  2.4 / 19.1.2005
  - nova fce CanClose() povoluje nebo zakazuje zaviraci krizek u okna
  2.2 / 25.12.2004
  - NotifyWhenIdle() je thread-safe
  - TIdleTask rozhybava uspanou frontu zprav ( viz. PostAppMessage() )
  2.1 / 20.12.2004
  - novy parametr NotifyWhenIdle(.., Singleton: Boolean = True) zastavi
    vytvoreni zpravy pokud uz je stejna zprava ve fronte
  2.0 / 16.12.2004
  - varianta NotifyWhenIdle() pro proceduru bez parametru
  1.9 / 30.9.2003
  - opraveno: lepsi podpora paletovych rezimu
  1.8 / 27.9.2003
  - opraveno: operace UnLockRedraw() nepodporovala paletove rezimy
  1.7 / 18.9.2003
  - opraveno: TWindowStyle.Apply zpusobovalo "Range check error"
  1.6 / 26.6.2003
  - kompatibilni s Delphi7
  1.5 / 17.4.2003
  - zprava naplanovana pomoci NotifyWhenIdle() se neposle, pokud je prijemce
    potomkem TComponent a je predcasne (pred prijetim zpravy) zrusen.
  1.4 / 28.3.2003
  - problem s procedurou spustenou pomoci NotifyWhenIdle() sice neovlivni
    dalsi zpracovani, ale uz se netutla
  1.3 / 3.12.2002
  - NotifyWhenIdle() vraci objekt, jehoz zrusenim se akce stornuje
  1.2 / 7.6.2002
  - soucastka TIdleTask se sama zrusi a spusti svou metodu Execute s
    pomoci Application.OnIdle, tj. kdyz neni zrovna co delat.
    TIdleTask-u je mozno vyrobit vic dopredu. Spusti se v poradi
    FirstInFirstOut.
  - procedure NotifyWhenIdle spusti udalost typu TNotifyEvent s pouzitim
    planovace TIdleTask
  1.1 / 27.2.2002
  - LockRedraw()
}
unit MiscUtils;

interface
uses Windows, Controls, Classes, SysUtils, Forms;

procedure Wait(yeap: boolean);
procedure LockUpdate(Handle: THandle);

procedure LockRedraw(Window: TWinControl);
procedure UnLockRedraw(Window: TWinControl);

procedure DisableControls(Parent: TWinControl; including: boolean);
procedure EnableControls(Parent: TWinControl; including: boolean);

procedure CenterWindow(Window: TWinControl; Parent: TComponent);
function ScreenSize(taskBar: boolean): TSize;
function ScreenRect(taskBar: boolean): TRect;
function WindowBorderSize(Window: TWinControl): TSize;

function NormalWindowRect(Window: TWinControl): TRect;

procedure PerformKey(Control: TWinControl; Key: word; Shift: TShiftState);

procedure CanClose(Window: TWinControl; Value: Boolean);

type
  TWindowStyle = class
  private
    FWindow: TWinControl;
  public
    Style, ExStyle: longint;
    constructor Create(AWindow: TWinControl);
    procedure Load;
    procedure Save;
    procedure Apply;
    procedure Assign(AStyle: TWindowStyle);
    property Window: TWinControl read FWindow write FWindow;
  end;

type
  TControlEnabler = class
  private
    FList: TList;
    org: boolean;
    Focuser: TWinControl;
  public
    constructor Create(Parent: TWinControl; including: boolean; value: boolean);
    destructor Destroy; override;
  end;

type
  TIdleTask = class(TComponent)
  protected
    State: (sWaiting, sRunning, sFinished);
    FOldOnIdle: TIdleEvent;
    FChain: TIdleTask;
    procedure OnIdle(Sender: TObject; var Done: boolean);
    procedure Activate;
    procedure Deactivate;
  public
    function GetUID: string; virtual; abstract;
    constructor Create;
    destructor Destroy; override;
    procedure Execute; virtual;
  end;

  { spusti udalost pomoci TIdleTask }
function NotifyWhenIdle(Sender: TObject; Event: TNotifyEvent; Singleton: Boolean = False): TComponent; overload;
function NotifyWhenIdle(Event: TThreadMethod; Singleton: Boolean = False): TComponent; overload;
function NotifyWhenIdle(Event: TProcedure; Singleton: Boolean = False): TComponent; overload;

{ spusti udalost pomoci TIdleTask, ale az po uplynuti 'Delay' }
function DelayedNotify(Sender: TObject; Event: TNotifyEvent; Delay: Cardinal): TComponent; overload;
function DelayedNotify(Event: TThreadMethod; Delay: Cardinal): TComponent; overload;
function DelayedNotify(Event: TProcedure; Delay: Cardinal): TComponent; overload;

implementation
uses Messages, SimpleTimer { TODO: pastni to dovnitr };

type
  TW = class(TWinControl)
  end;

procedure PerformKey(Control: TWinControl; Key: word; Shift: TShiftState);
var
  ch: char;
begin
  Control.SetFocus;
  TW(Control).KeyDown(Key, Shift);
  if Key <> 0 then begin
    if Key in [VK_RETURN, VK_TAB, VK_BACK, Ord('A')..Ord('Z'), Ord('0')..Ord('9')] then begin
      ch := chr(Key);
      TW(Control).KeyPress(ch)
    end;
    TW(Control).KeyUp(Key, Shift)
  end;
end;

procedure TWindowStyle.Assign;
begin
  FWindow := AStyle.FWindow;
  Style := AStyle.Style;
  ExStyle := AStyle.ExStyle
end;

constructor TWindowStyle.Create;
begin
  inherited Create;
  FWindow := AWindow
end;

procedure TWindowStyle.Apply;
var
  hwndAfter: HWND;
begin
  if (WS_EX_TOPMOST and ExStyle) <> 0 then
    hwndAfter := HWND_TOPMOST
  else
    hwndAfter := HWND_NOTOPMOST;
  SetWindowPos(Window.Handle, hwndAfter, 0, 0, 0, 0, SWP_NOMOVE or SWP_NOSIZE or SWP_NOCOPYBITS or SWP_NOACTIVATE or SWP_FRAMECHANGED)
end;

procedure TWindowStyle.Load;
begin
  Style := GetWindowLong(Window.Handle, GWL_STYLE);
  ExStyle := GetWindowLong(Window.Handle, GWL_EXSTYLE)
end;

procedure TWindowStyle.Save;
begin
  SetWindowLong(Window.Handle, GWL_STYLE, Style);
  SetWindowLong(Window.Handle, GWL_EXSTYLE, ExStyle);
end;

//----------------------------------------------------

constructor TControlEnabler.Create;

  procedure FlagControls(Parent: TWinControl; including: boolean; value: boolean);
  var
    i: integer;
    c: TControl;

    (*   procedure Extra(c:TControl);
       begin
        if c is TTabSheet then with TTabSheet(c) do begin
    //     Invalidate;Update
        end
       end; *)

    procedure Add(c: TControl);
    begin
      FList.Add(c);
    end;

  begin
    for i := 0 to Parent.ControlCount - 1 do begin
      c := Parent.Controls[i];
      if c.Enabled <> value then begin
        Add(c);
        c.Enabled := value;
        //     Extra(c)
      end;
      try
        if c is TWinControl then
          FlagControls(TWinControl(c), true, value)
      except
      end
    end;
    if including then begin
      if Parent.Enabled <> value then begin
        Add(Parent);
        Parent.Enabled := value;
        //     Extra(Parent)
      end
    end
  end;

begin
  FList := TList.Create;
  Focuser := Screen.ActiveControl;
  FlagControls(Parent, including, value);
  org := not value;
  Parent.Invalidate;
  UpdateWindow(Parent.Handle)
end;

destructor TControlEnabler.Destroy;
var
  i: integer;
begin
  for i := 0 to FList.Count - 1 do
    TControl(FList[i]).Enabled := org;
  FList.Free;
  if Assigned(Focuser) and Focuser.CanFocus then
    Focuser.SetFocus
end;

//----------------------------------------------------

procedure CanClose(Window: TWinControl; Value: Boolean);
var
  Mnu: HMENU;
begin
  Mnu := GetSystemMenu(Window.Handle, False);
  if Value then
    EnableMenuItem(Mnu, SC_CLOSE, MF_BYCOMMAND or MF_ENABLED)
  else
    EnableMenuItem(Mnu, SC_CLOSE, MF_BYCOMMAND or MF_GRAYED)
end;

//----------------------------------------------------

function NormalWindowRect;
var
  placement: TWindowPlacement;
begin
  placement.length := sizeof(placement);
  GetWindowPlacement(Window.Handle, @placement);
  Result := placement.rcNormalPosition
end;

//----------------------------------------------------

procedure FlagControls(Parent: TWinControl; including: boolean; value: boolean);
var
  i: integer;
begin
  for i := 0 to Parent.ControlCount - 1 do begin
    Parent.Controls[i].Enabled := value;
    try
      if Parent.Controls[i] is TWinControl then
        FlagControls(TWinControl(Parent.Controls[i]), true, value)
    except
    end
  end;
  if including then
    Parent.Enabled := value
end;

procedure DisableControls(Parent: TWinControl; including: boolean);
begin
  FlagControls(Parent, including, false)
end;

procedure EnableControls(Parent: TWinControl; including: boolean);
begin
  FlagControls(Parent, including, true)
end;

{$J+}

procedure Wait;
const
  counter: integer = 0;
  oldCursor: TCursor = 0;
begin
  if yeap then begin
    if counter = 0 then begin
      oldCursor := Screen.Cursor;
      Screen.Cursor := crHourGlass;
    end;
    inc(counter)
  end
  else begin
    if counter > 0 then dec(counter);
    if counter = 0 then
      if Screen.Cursor = crHourGlass then
        Screen.Cursor := oldCursor
  end
end;

procedure LockUpdate;
const
  counter: integer = 0;
  doUnlock: boolean = false;
begin
  if Handle <> 0 then begin
    if counter = 0 then begin
      doUnlock := IsWindowVisible(Handle);
      if doUnlock then LockWindowUpdate(Handle);
    end;
    inc(counter)
  end
  else begin
    if counter > 0 then dec(counter);
    if (counter = 0) and doUnlock then
      LockWindowUpdate(0)
  end
end;

function ScreenRect(taskBar: boolean): TRect;
begin
  if taskBar then
    SystemParametersInfo(spi_getworkarea, 0, @Result, 0)
  else begin
    Result.left := 0;
    Result.Right := GetSystemMetrics(SM_CXSCREEN);
    Result.Top := 0;
    Result.Bottom := GetSystemMetrics(SM_CYSCREEN)
  end
end;

function ScreenSize;
begin
  with Result, ScreenRect(taskbar) do begin
    cx := Right - Left;
    cy := Bottom - Top
  end
end;

procedure CenterWindow;
var
  r: TRect;
  p: TPoint;
begin
  if Assigned(Parent) then begin
    if Parent is TScreen then
      r := ScreenRect(false)
    else
      if Parent is TWinControl then
        with TWinControl(Parent) do begin
          GetWindowRect(Handle, r);
          //    OffsetRect(r,Left,Top)
        end
      else
        exit
  end
  else
    r := ScreenRect(true);

  with Window do begin
    p.x := r.Left + ((r.right - r.left) - Width) div 2;
    p.y := r.Top + ((r.bottom - r.top) - Height) div 2
  end;
  p := Window.ScreenToClient(p);
  if false {Window.HandleAllocated} then begin
    SetWindowPos(Window.Handle, 0, p.x, p.y, 0, 0, SWP_NOSIZE or SWP_NOZORDER {or SWP_NOREDRAW})
  end
  else begin
    Window.Left := Window.Left + p.x;
    Window.Top := Window.Top + p.y
  end
end;

function WindowBorderSize;
var
  w, c: TRect;
  { pos:TWindowPos;
    p:TNCCalcSizeParams; }
begin
  GetWindowRect(Window.Handle, w);
  GetClientRect(Window.Handle, c);
  {  p.rgrc[0]:=w;
    p.rgrc[1]:=w;
    p.rgrc[2]:=c;
    FillChar(pos,sizeof(pos),0);
    pos.hwnd:=Window.Handle;
    pos.flags:=SWP_NOMOVE or SWP_NOSIZE or SWP_NOZORDER or SWP_NOREDRAW or SWP_NOACTIVATE;
    p.lppos:=@pos;
    SendMessage(Window.Handle,WM_NCCALCSIZE,1,integer(@p));
    w:=p.rgrc[0];
    c:=p.rgrc[2];}
  Result.cx := (w.Right - w.Left) - (c.Right - c.Left);
  Result.cy := (w.Bottom - w.Top) - (c.Bottom - c.Top)
end;

var
  locks, counters: TList;

procedure LockReDraw;
var
  i: integer;
begin
  if not Assigned(Window) then exit;

  i := locks.IndexOf(Window);
  if i < 0 then begin
    locks.Add(Window);
    counters.Add(pointer(1));
    Window.Perform(WM_SETREDRAW, WParam(False), 0)
  end
  else
    counters[i] := pointer(integer(counters[i]) + 1)
end;

procedure UnLockRedraw;
var
  i: integer;
  cnt: integer;
  wr, cr: TRect;
  p: TPoint;
begin
  if not Assigned(Window) then exit;

  i := locks.IndexOf(Window);
  if i < 0 then exit;

  cnt := integer(counters[i]) - 1;
  if cnt = 0 then begin
    locks.Delete(i);
    counters.Delete(i);
    Window.Perform(WM_SETREDRAW, WParam(True), 0);
    TW(Window).PaletteChanged(true);
    Windows.RedrawWindow(Window.Handle, nil, 0, RDW_INVALIDATE or RDW_FRAME or RDW_ALLCHILDREN or RDW_UPDATENOW);
    {

    Windows.GetWindowRect(Window.Handle,wr);
    Windows.GetClientRect(Window.Handle,cr);
    p.x:=cr.Left;
    p.y:=cr.Top;
    Windows.ClientToScreen(Window.Handle,p);

    cr.Left:=wr.Left-p.x;
    cr.Top:=wr.Top-p.y;
    cr.Right:=cr.Left+(wr.Right-wr.Left);
    cr.Bottom:=cr.Top+(wr.Bottom-wr.Top);

    Windows.InvalidateRect(Window.Handle,@cr,true);
    Windows.UpdateWindow(Window.Handle) }
  end
  else
    counters[i] := pointer(cnt)
end;

type
  TIdleNotify = class;
  TIdleNotifyClass = class of TIdleNotify;
  TIdleNotify = class(TIdleTask)
  protected
    FRecipient: TComponent;
    FEvent: TNotifyEvent;
    FSender: TObject;
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create(ASender: TObject; AnEvent: TNotifyEvent); virtual;
    procedure Execute; override;
    function GetUID: string; override;
    class function GenUID(ASender: TObject; AnEvent: TNotifyEvent): string;
  end;

function BufToHex(const Buf; BufSize: Integer): string;
var
  I: Integer;
  P: PByteArray;
begin
  Result := '';
  P := @Buf;
  for I := 1 to BufSize do
    Result := Result + IntToHex(P^[I - 1], 2)
end;

function TIdleNotify.GetUID: string;
begin
  Result := GenUID(FSender, FEvent)
end;

class function TIdleNotify.GenUID(ASender: TObject; AnEvent: TNotifyEvent): string;
var
  Buf: record
    ASender: TObject;
    AnEvent: TNotifyEvent
  end;
begin
  Buf.ASender := ASender;
  Buf.AnEvent := AnEvent;
  Result := BufToHex(Buf, SizeOf(Buf))
end;

procedure TIdleNotify.Notification;
begin
  if ((AComponent = FRecipient) or (AComponent = FSender)) and (Operation = opRemove) then begin
    FRecipient := nil;
    FEvent := nil
  end;
  inherited
end;

constructor TIdleNotify.Create;
var
  method: TMethod absolute AnEvent;
begin
  if (method.Data <> nil) and (TObject(method.Data) is TComponent) then begin
    FRecipient := TComponent(method.Data);
    FRecipient.FreeNotification(Self)
  end;
  if (ASender <> nil) and (ASender is TComponent) then
    TComponent(ASender).FreeNotification(Self);
  FEvent := AnEvent;
  FSender := ASender;
  inherited Create
end;

procedure TIdleNotify.Execute;
begin
  if Assigned(FEvent) then
    FEvent(FSender)
end;

type
  TSimpleNotify = class(TIdleNotify)
  public
    constructor Create(AnEvent: TThreadMethod); overload;
    constructor Create(ASender: TObject; AnEvent: TNotifyEvent); overload; override;
    procedure Execute; override;
    function GetUID: string; override;
    class function GenUID(AnEvent: TThreadMethod): string;
  end;

function TSimpleNotify.GetUID: string;
begin
  Result := GenUID(TThreadMethod(FEvent))
end;

class function TSimpleNotify.GenUID(AnEvent: TThreadMethod): string;
var
  Buf: record
    AnEvent: TThreadMethod
  end;
begin
  Buf.AnEvent := AnEvent;
  Result := BufToHex(Buf, SizeOf(Buf))
end;

constructor TSimpleNotify.Create(ASender: TObject; AnEvent: TNotifyEvent);
begin
  inherited Create(ASender, AnEvent)
end;

constructor TSimpleNotify.Create(AnEvent: TThreadMethod);
begin
  inherited Create(nil, TNotifyEvent(AnEvent))
end;

procedure TSimpleNotify.Execute;
begin
  if Assigned(FEvent) then
    TThreadMethod(FEvent)
end;

type
  TSimpleProcNotify = class(TIdleNotify)
  public
    constructor Create(AnEvent: TProcedure); overload;
    constructor Create(ASender: TObject; AnEvent: TNotifyEvent); overload; override;
    procedure Execute; override;
    function GetUID: string; override;
    class function GenUID(AnEvent: TProcedure): string;
  end;

function TSimpleProcNotify.GetUID: string;
begin
  Result := GenUID(TProcedure(TMethod(FEvent).Code))
end;

class function TSimpleProcNotify.GenUID(AnEvent: TProcedure): string;
var
  Buf: record
    AnEvent: TProcedure
  end;
begin
  Buf.AnEvent := AnEvent;
  Result := BufToHex(Buf, SizeOf(Buf))
end;

constructor TSimpleProcNotify.Create(ASender: TObject; AnEvent: TNotifyEvent);
begin
  inherited Create(ASender, AnEvent)
end;

constructor TSimpleProcNotify.Create(AnEvent: TProcedure);
var
  method: TMethod;
begin
  method.Code := @AnEvent;
  method.Data := nil;
  inherited Create(nil, TNotifyEvent(method))
end;

procedure TSimpleProcNotify.Execute;
begin
  if Assigned(FEvent) then
    TProcedure(TMethod(FEvent).Code)
end;

type
  TTaskKiller = class(TComponent)
  protected
    task: TIdleTask;
  public
    constructor Create(ATask: TIdleTask);
    destructor Destroy; override;
  end;

constructor TTaskKiller.Create;
begin
  inherited Create(ATask);
  task := ATask
end;

destructor TTaskKiller.Destroy;
begin
  if task = Owner then
    if task.State in [sWaiting] then begin
      Owner.RemoveComponent(Self);
      task.Free
    end;
  inherited
end;

// vrati False kdyz uz je zaregistrovany nejaky TIdleTask s timtez UID
function NoNotify(const UID: string): Boolean; forward;

var
  NotifyLock: TRTLCriticalSection;

procedure EnterNotify;
begin
  EnterCriticalSection(NotifyLock)
end;

procedure LeaveNotify;
begin
  LeaveCriticalSection(NotifyLock)
end;

function NotifyWhenIdle(Sender: TObject; Event: TNotifyEvent; Singleton: Boolean): TComponent;
begin
  EnterNotify;
  try
    if (not Singleton) or NoNotify(TIdleNotify.GenUID(Sender, Event)) then
      Result := TTaskKiller.Create(TIdleNotify.Create(Sender, Event))
    else
      Result := nil
  finally
    LeaveNotify
  end
end;

function NotifyWhenIdle(Event: TThreadMethod; Singleton: Boolean): TComponent;
begin
  EnterNotify;
  try
    if (not Singleton) or NoNotify(TSimpleNotify.GenUID(Event)) then
      Result := TTaskKiller.Create(TSimpleNotify.Create(Event))
    else
      Result := nil
  finally
    LeaveNotify
  end
end;

function NotifyWhenIdle(Event: TProcedure; Singleton: Boolean): TComponent;
begin
  EnterNotify;
  try
    if (not Singleton) or NoNotify(TSimpleProcNotify.GenUID(Event)) then
      Result := TTaskKiller.Create(TSimpleProcNotify.Create(Event))
    else
      Result := nil
  finally
    LeaveNotify
  end
end;

type
  TDelayer = class(TComponent)
  private
    FWaker: TSimpleTimer;
    FSender: TObject;
    FMethod: TMethod;
    Factory: TIdleNotifyClass;
    procedure OnTimer(Sender: TObject);
    procedure Execute;
  public
    constructor Create(AFactory: TIdleNotifyClass);
    destructor Destroy; override;
    procedure Start(Delay: Cardinal);
  end;

constructor TDelayer.Create(AFactory: TIdleNotifyClass);
begin
  inherited Create(Application);
  Factory := AFactory
end;

destructor TDelayer.Destroy;
var
  Task: TIdleTask;
begin
  if Assigned(Owner) and (Owner is TIdleTask) then begin
    Task := TIdleTask(Owner);
    if Task.State in [sWaiting] then begin
      Owner.RemoveComponent(Self);
      Task.Free
    end
  end;
  FWaker.Free;
  inherited
end;

procedure TDelayer.OnTimer(Sender: TObject);
begin
  FWaker.Enabled := False;
  Execute
end;

procedure TDelayer.Execute;
var
  Task: TIdleTask;
begin
  if (FMethod.Code <> nil) and (Factory <> nil) then begin
    Task := Factory.Create(FSender, TNotifyEvent(FMethod));
    // ted se prepojime tak, aby TDelayer dal fungoval ve funkci TTaskKiller-u
    if Assigned(Owner) then
      Owner.RemoveComponent(Self);
    Task.InsertComponent(Self);
  end
  else
    // TODO: nemelo by se to zpozdit ?
    Free
end;

procedure TDelayer.Start(Delay: Cardinal);
begin
  if Delay = 0 then
    Execute
  else begin
    FWaker := TSimpleTimer.Create;
    FWaker.Interval := Delay;
    FWaker.OnTimer := OnTimer;
    FWaker.Enabled := True
  end
end;

function DelayedNotify(Sender: TObject; Event: TNotifyEvent; Delay: Cardinal): TComponent;
var
  Delayer: TDelayer;
begin
  if Delay = 0 then
    Result := NotifyWhenIdle(Sender, Event)
  else begin
    EnterNotify;
    try
      Delayer := TDelayer.Create(TIdleNotify);
      Result := Delayer;
      Delayer.FSender := Sender;
      Delayer.FMethod := TMethod(Event);
      Delayer.Start(Delay)
    finally
      LeaveNotify
    end
  end
end;

function DelayedNotify(Event: TThreadMethod; Delay: Cardinal): TComponent;
var
  Delayer: TDelayer;
begin
  if Delay = 0 then
    Result := NotifyWhenIdle(Event)
  else begin
    EnterNotify;
    try
      Delayer := TDelayer.Create(TSimpleNotify);
      Result := Delayer;
      Delayer.FMethod := TMethod(Event);
      Delayer.Start(Delay)
    finally
      LeaveNotify
    end
  end
end;

function DelayedNotify(Event: TProcedure; Delay: Cardinal): TComponent;
var
  Delayer: TDelayer;
begin
  if Delay = 0 then
    Result := NotifyWhenIdle(Event)
  else begin
    EnterNotify;
    try
      Delayer := TDelayer.Create(TSimpleProcNotify);
      Result := Delayer;
      Delayer.FMethod.Code := @Event;
      Delayer.Start(Delay)
    finally
      LeaveNotify
    end
  end
end;

//-------------------------------------------

var
  curTask: TIdleTask = nil;
  Idlers: Integer = 0;

constructor TIdleTask.Create;
var
  Parent: TIdleTask;
begin
  Inc(Idlers);
  // TODO:
  // Assert(Idlers < 20);

  inherited Create(Application);
  State := sWaiting;

  if Assigned(curTask) then begin
    Parent := curTask;
    while Assigned(Parent.FChain) do
      Parent := Parent.FChain;
    Parent.FChain := Self
  end
  else
    Activate
end;

function NoNotify(const UID: string): Boolean;
var
  Tsk: TIdleTask;
begin
  Result := True;
  Tsk := curTask;
  while Assigned(Tsk) do begin
    if Tsk.GetUID = UID then begin
      if Tsk.State in [sWaiting] then
        Result := False;
      exit
    end;
    Tsk := Tsk.FChain
  end
end;

procedure TIdleTask.Activate;
begin
  EnterNotify;
  FOldOnIdle := Application.OnIdle;
  Application.OnIdle := OnIdle;
  curTask := Self;
  LeaveNotify;
  // TODO:
  PostAppMessage(MainThreadID, WM_NULL, 0, 0)
end;

procedure TIdleTask.Deactivate;
begin
  EnterNotify;
  try
    if curTask <> Self then exit;
    Application.OnIdle := FOldOnIdle;
    curTask := nil;
    if Assigned(FChain) then
      FChain.Activate
  finally
    LeaveNotify
  end
end;

destructor TIdleTask.Destroy;
var
  Parent: TIdleTask;
begin
  Dec(Idlers);
  Deactivate;

  EnterNotify;
  if Assigned(curTask) then begin
    { prohledame retez a odpojime se z neho }
    Parent := curTask;
    repeat
      if Parent.FChain = Self then begin
        Parent.FChain := FChain;
        break
      end;
      Parent := Parent.FChain
    until Parent = nil
  end;
  LeaveNotify;

  inherited
end;

procedure TIdleTask.OnIdle;
begin
  EnterNotify; // TODO: ? no muze to sem vbehnout z jineho vlakna ?
  Done := False; // TODO: ?
  Application.OnIdle := nil;
  LeaveNotify;
  try
    State := sRunning;
    Execute
  except
    // TODO:
    Application.HandleException(Self)
  end;
  State := sFinished;
  Deactivate;
  Free
end;

procedure TIdleTask.Execute;
begin
end;

//-------------------------------------------

initialization
  locks := TList.Create;
  counters := TList.Create;
  InitializeCriticalSection(NotifyLock)
finalization
  locks.Free;
  counters.Free;
  DeleteCriticalSection(NotifyLock)
end.

