{ VERSION: 1.1 }
{ Definuje pomocn pistupova k publikovanm vlastnostem objektu }
unit TypInfoUtils;

interface
uses
  TypInfo;

type
  TPropBinder = class
  protected
    FInstance: TObject;
    FPropInfo: PPropInfo;
    Child: TPropBinder;
  public
    property Instance: TObject read FInstance;
    property PropInfo: PPropInfo read FPropInfo;
  public
    function BindPropInfo: PPropInfo;
    function Bind: TPropBinder; overload;
    function Bind(Instance: TObject): TPropBinder; overload;
    destructor Destroy; override;
  end;

function CreateBinder(TypeInfo: PTypeInfo; const PropertyName: string): TPropBinder; overload;
function CreateBinder(Instance: TObject; const PropertyName: string): TPropBinder; overload;
function CreateBinder(AClass: TClass; const PropertyName: string): TPropBinder; overload;

implementation
uses
  Classes, SysUtils;

function CreateBinder(Instance: TObject; const PropertyName: string): TPropBinder;
begin
  if Assigned(Instance) then begin
    Result := CreateBinder(PTypeInfo(Instance.ClassInfo), PropertyName);
    if Assigned(Instance) then
      Result.FInstance := Instance
  end else
    Result := nil
end;

function CreateBinder(AClass: TClass; const PropertyName: string): TPropBinder;
begin
  if Assigned(AClass) then
    Result := CreateBinder(PTypeInfo(AClass.ClassInfo), PropertyName)
  else
    Result := nil
end;

function CreateBinder(TypeInfo: PTypeInfo; const PropertyName: string): TPropBinder;

  function CreateSimpleBinder(TypeInfo: PTypeInfo; const PropertyName: string): TPropBinder;
  var
    PropInfo: PPropInfo;
  begin
    Result := nil;
    if TypeInfo <> nil then begin
      PropInfo := GetPropInfo(TypeInfo, PropertyName);
      if Assigned(PropInfo) then begin
        Result := TPropBinder.Create;
        Result.FPropInfo := PropInfo
      end
    end
  end;

var
  Path: TStrings;
  P: Integer;
  Binder: TPropBinder;
  Parent: TPropBinder;
  Child: TPropBinder;
begin
  if Pos('.', PropertyName) = 0 then
    Result := CreateSimpleBinder(TypeInfo, PropertyName)
  else begin
    Path := TStringList.Create;
    try
      Path.Text := StringReplace(PropertyName, '.', #13#10, [rfReplaceAll]);

      Binder := nil;
      Parent := nil;

      for P := 0 to Path.Count - 1 do begin
        Child := CreateBinder(TypeInfo, Path[P]);
        if not Assigned(Child) then begin
          FreeAndNil(Binder);
          break
        end;
        if not Assigned(Binder) then begin
          Binder := Child;
          Parent := Binder
        end else begin
          Parent.Child := Child;
          Parent := Child
        end;
        if Child.FPropInfo^.PropType^^.Kind = tkClass then
          TypeInfo := Child.FPropInfo^.PropType^
        else
          TypeInfo := nil;
      end
    finally
      Path.Free
    end;
    Result := Binder
  end
end;

function TPropBinder.Bind(Instance: TObject): TPropBinder;
begin
  FInstance := Instance;
  Result := Bind
end;

function TPropBinder.Bind: TPropBinder;
begin
  if not Assigned(FInstance) then
    Result := nil
  else begin
    if Assigned(Child) then
      Result := Child.Bind(TObject(GetOrdProp(FInstance, FPropInfo)))
    else
      Result := Self
  end
end;

function TPropBinder.BindPropInfo: PPropInfo;
begin
  if Assigned(Child) then
    Result := Child.BindPropInfo
  else
    Result := PropInfo
end;

destructor TPropBinder.Destroy;
begin
  Child.Free;
  inherited
end;

end.
