{ VERSION: 1.5 }
{ Strom objekt s pamt propojen na njak jin objekt a jeho 'property' }
{ 1.5./ 8.6.2005
  - opraveno: po odpojen pipojenho objektu se dl pouvala pvodn ( u neplatn ) hodnota
  1.4 / 12.5.2005
  - zveejnna funkce TLinkedStringColumnDescriptor.GetValue(Node: TTreeNode): Variant
  1.3 / 9.5.2005
  - opraveno: datum typu NULL se vyhodnocoval jako xxyy1998
  - opraveno: pi zruen pipojench objekt dochzelo pi ruen
    stromu k 'Access Violation'. pro objekty typu TComponent se odpojen
    automaticky detekuje (viz. TLinkedStringNodeManager.Guard)
  1.2 / 2.5.2005
  - v jednom dku me bt zobrazeno vc rznch objekt. viz. Data[]
  - azen sloupce se d nastavit udlost TLinkedStringColumnDescriptor.OnCompare()
  - TLinkedStringColumnDescriptor.Link() podporuje vlastnosti typu tkVariant
  - pi napojen sloupce na vlastnost metodou Link() se nastav vchoz jmno sloupce podle
    jmna napojen vlastnosti
  1.1 / 26.4.2005
}
unit uLinkedStringTree;

interface
uses
  Classes, uObjectTree, uStringTree, TypInfoUtils;

type
  TCustomLinkedStringNode = class;
  TLinkedStringNodeManager = class(TStringNodeManager)
  private
    FGuard: TObject;
  protected
    procedure Guard(Node: TCustomLinkedStringNode; Data: TComponent; Active: Boolean);
  protected
    function CreateDescriptors: TNodeDescriptors; override;
  public
    destructor Destroy; override;
  end;

  TGetValue = class
  public
    function GetValue(Binder: TPropBinder): Variant; virtual;
  end;
  TColumnCompareEvent = function (Sender: TColumnDescriptor; Node1: TTreeNode; Data1: TObject; Node2: TTreeNode; Data2: TObject): Integer of object;
  TLinkedStringColumnDescriptor = class(TColumnDescriptor)
  private
    FOnCompare: TColumnCompareEvent;
    FBinder: TPropBinder;
    FGetValue: TGetValue;
    FDataIndex: Integer;
  protected
    function Bind(Node: TCustomLinkedStringNode): TPropBinder;
  public
    function GetValue(Node: TCustomLinkedStringNode): Variant;
  public
    function Compare(Node1, Node2: TTreeNode): Integer; override;
  public
    function IsLinked: Boolean; overload;
    function IsLinked(DataIndex: Integer): Boolean; overload;
    procedure Assign(Source: TPersistent); override;
    destructor Destroy; override;
    procedure Link(AClass: TClass; const PropertyName: string; DataIndex: Integer = 0);
    property OnCompare: TColumnCompareEvent read FOnCompare write FOnCompare;
  end;

  TLinkedStringNodeDescriptor = class(TStringNodeDescriptor)
  protected
    function CreateColumns: TColumnDescriptors; override;
  end;

  TCustomLinkedStringNode = class(TStringNode)
  private
    FData: array of TObject;
    function GetData(Index: Integer): TObject;
    procedure SetData(Index: Integer; Value: TObject);
    procedure UnLink(Data: TObject);
  protected
    Updating: Boolean;
    procedure SetValueAsCode(Index: Integer; const Value: string); override;
  protected
    function GetColumnData(Index: Integer): TObject;
    procedure SetColumnData(Index: Integer; Value: TObject);
    procedure ColumnChanged(Index: Integer); virtual;
    procedure LinkData(DataIndex: Integer; Data: TObject); virtual;
    procedure UnLinkData(DataIndex: Integer; Data: TObject); virtual;
  public
    destructor Destroy; override;
  protected
    property Data[Index: Integer]: TObject read GetData write SetData;
    property ColumnData[Index: Integer]: TObject read GetColumnData write SetColumnData;
  end;

  TLinkedStringNode = class(TCustomLinkedStringNode)
  public
    property Data;
  end;

implementation
uses
  TypInfo, SysUtils, Variants, DateUtils;

// TGetValue -----------------------------------------------------------------

function TGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := Unassigned
end;

type
  TBaseGetValue = class(TGetValue)
  protected
    PropInfo: PPropInfo;
  public
    constructor Create(PropInfo: PPropInfo);
  end;

constructor TBaseGetValue.Create(PropInfo: PPropInfo);
begin
  Self.PropInfo := PropInfo;
  inherited Create;
end;

type
  TIntegerGetValue = class(TBaseGetValue)
  private
    IntToIdent: TIntToIdent;
  public
    procedure AfterConstruction; override;
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

procedure TIntegerGetValue.AfterConstruction;
begin
  inherited;
  IntToIdent := FindIntToIdent(PropInfo^.PropType^);
end;

function TIntegerGetValue.GetValue(Binder: TPropBinder): Variant;
var
  Value: Integer;
  Text: string;
begin
  Value := GetOrdProp(Binder.Instance, Binder.PropInfo);
  if Assigned(IntToIdent) and IntToIdent(Value, Text) then
    Result := Text
  else
    Result := Value
end;

type
  TStringGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TStringGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := GetStrProp(Binder.Instance, Binder.PropInfo)
end;

type
  TWideStringGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TWideStringGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := GetWideStrProp(Binder.Instance, Binder.PropInfo)
end;

type
  TCharGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TCharGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := Char(GetOrdProp(Binder.Instance, Binder.PropInfo))
end;

type
  TWideCharGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TWideCharGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := WideChar(GetOrdProp(Binder.Instance, Binder.PropInfo))
end;

type
  TEnumGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TEnumGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := GetEnumProp(Binder.Instance, Binder.PropInfo)
end;

type
  TFloatGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TFloatGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := GetFloatProp(Binder.Instance, Binder.PropInfo)
end;

type
  TSetGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TSetGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := GetSetProp(Binder.Instance, Binder.PropInfo)
end;

type
  TDateTimeGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TDateTimeGetValue.GetValue(Binder: TPropBinder): Variant;
var
  Value: TDateTime;
begin
  Value := GetFloatProp(Binder.Instance, Binder.PropInfo);
  if Value = 0 then
    Result := Unassigned
  else
    Result := Value
end;

type
  TVariantGetValue = class(TBaseGetValue)
  public
    function GetValue(Binder: TPropBinder): Variant; override;
  end;

function TVariantGetValue.GetValue(Binder: TPropBinder): Variant;
begin
  Result := GetVariantProp(Binder.Instance, Binder.PropInfo)
end;

// TLinkedStringColumnDescriptor ---------------------------------------------

destructor TLinkedStringColumnDescriptor.Destroy;
begin
  FreeAndNil(FBinder);
  FreeAndNil(FGetValue);
  inherited
end;

function TLinkedStringColumnDescriptor.Bind(Node: TCustomLinkedStringNode): TPropBinder;
begin
  if Assigned(FBinder) then
    Result := FBinder.Bind(Node.Data[FDataIndex])
  else
    Result := nil
end;

function TLinkedStringColumnDescriptor.GetValue(Node: TCustomLinkedStringNode): Variant;
var
  Binder: TPropBinder;
begin
  Binder := Bind(Node);
  if Assigned(Binder) then
    Result := FGetValue.GetValue(Binder)
  else
    Result := Unassigned
end;

procedure TLinkedStringColumnDescriptor.Link(AClass: TClass; const PropertyName: string; DataIndex: Integer);
var
  PropInfo: PPropInfo;
  TypeInfo: PTypeInfo;
  TypeName: string;
begin
  FreeAndNil(FBinder);
  FreeAndNil(FGetValue);
  FDataIndex := DataIndex;

  FBinder := CreateBinder(AClass, PropertyName);
  if Assigned(FBinder) then begin
    PropInfo := FBinder.BindPropInfo;
    TypeInfo := PropInfo^.PropType^;
    TypeName := TypeInfo^.Name;
    case TypeInfo^.Kind of
    tkInteger:
      FGetValue := TIntegerGetValue.Create(PropInfo);

    tkChar:
      FGetValue := TCharGetValue.Create(PropInfo);

    tkEnumeration:
      FGetValue := TEnumGetValue.Create(PropInfo);

    tkFloat:
      if TypeInfo = System.TypeInfo(TDateTime) then
        FGetValue := TDateTimeGetValue.Create(PropInfo)
      else
        FGetValue := TFloatGetValue.Create(PropInfo);

    tkString, tkLString:
      FGetValue := TStringGetValue.Create(PropInfo);

    tkSet:
      FGetValue := TSetGetValue.Create(PropInfo);

    tkWChar:
      FGetValue := TWideCharGetValue.Create(PropInfo);

    tkWString:
      FGetValue := TWideStringGetValue.Create(PropInfo);

    tkVariant:
      FGetValue := TVariantGetValue.Create(PropInfo);

    else
      // TODO:
    end
  end;
  if not Assigned(FGetValue) then
    FGetValue := TGetValue.Create;

  // nastavime vychozi id a popisek  
  if Name = '' then begin
    if FDataIndex = 0 then
      Name := PropertyName
    else
      Name := PropertyName + IntToStr(FDataIndex)
  end;
  if Caption = '' then
    Caption := PropertyName
end;

function TLinkedStringColumnDescriptor.IsLinked: Boolean;
begin
  Result := Assigned(FBinder)
end;

function TLinkedStringColumnDescriptor.IsLinked(DataIndex: Integer): Boolean;
begin
  Result := Assigned(FBinder) and (FDataIndex = DataIndex)
end;

function TLinkedStringColumnDescriptor.Compare(Node1, Node2: TTreeNode): Integer;
begin
  if Assigned(FOnCompare) then begin
    if IsLinked then
      Result := FOnCompare(Self, Node1, TCustomLinkedStringNode(Node1).Data[FDataIndex], Node2, TCustomLinkedStringNode(Node2).Data[FDataIndex])
    else
      Result := FOnCompare(Self, Node1, nil, Node2, nil)
  end else
    Result := inherited Compare(Node1, Node2);
end;

procedure TLinkedStringColumnDescriptor.Assign(Source: TPersistent);
var
  Col: TLinkedStringColumnDescriptor absolute Source;
begin
  if Source is TLinkedStringColumnDescriptor then begin
    inherited;
    OnCompare := Col.OnCompare
  end else
    inherited
end;

// TLinkedStringNodeDescriptor -----------------------------------------------

function TLinkedStringNodeDescriptor.CreateColumns: TColumnDescriptors;
begin
  Result := TColumnDescriptors.Create(TLinkedStringColumnDescriptor);
end;

// TCustomLinkedStringNode ---------------------------------------------------------

procedure TCustomLinkedStringNode.LinkData(DataIndex: Integer; Data: TObject);
var
  Index: Integer;
  Col: TLinkedStringColumnDescriptor;
begin
  if Data is TComponent then begin
    Assert(Manager <> nil);
    TLinkedStringNodeManager(Manager).Guard(Self, TComponent(Data), True);
  end;
  for Index := 0 to Descriptor.Columns.Count - 1 do begin
    Col := TLinkedStringColumnDescriptor(Descriptor.Columns[Index]);
    if Col.IsLinked(DataIndex) then
      Values[Index] := Col.GetValue(Self)
  end
end;

procedure TCustomLinkedStringNode.UnLinkData(DataIndex: Integer; Data: TObject);
var
  Index: Integer;
  Col: TLinkedStringColumnDescriptor;
begin
  if Data is TComponent then begin
    Assert(Manager <> nil);
    TLinkedStringNodeManager(Manager).Guard(Self, TComponent(Data), False);
  end;
  for Index := 0 to Descriptor.Columns.Count - 1 do begin
    Col := TLinkedStringColumnDescriptor(Descriptor.Columns[Index]);
    if Col.IsLinked(DataIndex) then
      Values[Index] := Unassigned
  end
end;

procedure TCustomLinkedStringNode.UnLink(Data: TObject);
var
  I: Integer;
begin
  for I := 0 to Length(FData) - 1 do
    if FData[I] = Data then
      Self.Data[I] := nil
end;

function TCustomLinkedStringNode.GetData(Index: Integer): TObject;
begin
  if Index < Length(FData) then
    Result := FData[Index]
  else
    Result := nil
end;

procedure TCustomLinkedStringNode.SetData(Index: Integer; Value: TObject);
var
  OldUpdating: Boolean;
  I: Integer;
begin
  OldUpdating := Updating;
  Updating := True;
  try
    if Data[Index] <> nil then
      UnLinkData(Index, Data[Index]);

    if Index >= Length(FData) then begin
      // natahneme pole
      I := Length(FData);
      SetLength(FData, Index + 1);
      while I < Length(FData) - 1 do begin
        FData[I] := nil;
        Inc(I)
      end
    end;

    FData[Index] := Value;

    if Assigned(Value) then
      LinkData(Index, Value)
  finally
    Updating := OldUpdating
  end
end;

procedure TCustomLinkedStringNode.SetValueAsCode(Index: Integer; const Value: string);
var
  Changed: Boolean;
begin
  Changed := (not Updating) and (ColumnData[Index] <> nil) and (GetValueAsCode(Index) <> Value);
  inherited SetValueAsCode(Index, Value);
  if Changed then
    ColumnChanged(Index)
end;

function TCustomLinkedStringNode.GetColumnData(Index: Integer): TObject;
begin
  with TLinkedStringColumnDescriptor(Descriptor.Columns[Index]) do
    if IsLinked then
      Result := Data[FDataIndex]
    else
      Result := nil
end;

procedure TCustomLinkedStringNode.SetColumnData(Index: Integer; Value: TObject);
begin
  with TLinkedStringColumnDescriptor(Descriptor.Columns[Index]) do
    if IsLinked then
      Data[FDataIndex] := Value
end;

procedure TCustomLinkedStringNode.ColumnChanged(Index: Integer);
var
  Binder: TPropBinder;
begin
  Binder := TLinkedStringColumnDescriptor(Descriptor.Columns[Index]).Bind(Self);
  if Assigned(Binder) then
    // TODO: posli hodnotu do objektu
    ;
end;

destructor TCustomLinkedStringNode.Destroy;
var
  I: Integer;
begin
  for I := Length(FData) - 1 downto 0 do
    Data[I] := nil;
  inherited
end;

// TLinkedStringNodeManager -------------------------------------------------

function TLinkedStringNodeManager.CreateDescriptors: TNodeDescriptors;
begin
  Result := TNodeDescriptors.Create(TLinkedStringNodeDescriptor);
end;

type
  TObjectList = class(TStringList)
  private
    function GetTag(Index: Integer): Integer;
    procedure SetTag(Index: Integer; const Value: Integer);
    function GetItem(Index: Integer): TObject;
  protected
    function CompareStrings(const S1, S2: string): Integer; override;
  public
    constructor Create;
    function IndexOf(Item: TObject): Integer; overload;
    function IndexOf(const Key: string): Integer; overload;
    function Add(Item: TObject): Integer; overload;
    function Add(const Key: string): Integer; overload;
    property Tags[Index: Integer]: Integer read GetTag write SetTag;
    property Items[Index: Integer]: TObject read GetItem; default;
  end;

constructor TObjectList.Create;
begin
  inherited Create;
  Sorted := True
end;

function TObjectList.CompareStrings(const S1, S2: string): Integer;
begin
  Result := PInteger(@S1[1])^ - PInteger(@S2[1])^
end;

function ObjectToKey(Item: TObject): string;
begin
  SetLength(Result, SizeOf(Integer));
  PInteger(@Result[1])^ := Integer(Item)
end;

function KeyToObject(const Key: string): TObject;
begin
  Result := TObject(PInteger(@Key[1])^)
end;

function TObjectList.GetItem(Index: Integer): TObject;
begin
  Result := KeyToObject(Strings[Index])
end;

function TObjectList.IndexOf(Item: TObject): Integer;
begin
  Result := inherited IndexOf(ObjectToKey(Item))
end;

function TObjectList.IndexOf(const Key: string): Integer;
begin
  Result := inherited IndexOf(Key)
end;

function TObjectList.Add(Item: TObject): Integer;
begin
  Result := inherited Add(ObjectToKey(Item))
end;

function TObjectList.Add(const Key: string): Integer;
begin
  Result := inherited Add(Key)
end;

function TObjectList.GetTag(Index: Integer): Integer;
begin
  Result := Integer(Objects[Index])
end;

procedure TObjectList.SetTag(Index: Integer; const Value: Integer);
begin
  Objects[Index] := TObject(Value)
end;

type
  TGuard = class(TComponent)
  private
    FNodes: TObjectList;
  protected
    procedure Notification(AComponent: TComponent; Operation: TOperation); override;
  public
    constructor Create;
    destructor Destroy; override;
    procedure Guard(Node: TCustomLinkedStringNode; Data: TComponent; Active: Boolean);
  end;

constructor TGuard.Create;
begin
  inherited Create(nil);
  FNodes := TObjectList.Create
end;

procedure TGuard.Guard(Node: TCustomLinkedStringNode; Data: TComponent; Active: Boolean);
var
  I: Integer;
  Key: string;
begin
  Key := ObjectToKey(Node);
  if Active then begin
    I := FNodes.IndexOf(Key);
    if I < 0 then
      FNodes.Add(Key)
    else
      FNodes.Tags[I] := FNodes.Tags[I] + 1;
    Data.FreeNotification(Self)
  end else begin
    I := FNodes.IndexOf(Key);
    if I >= 0 then begin
      if FNodes.Tags[I] = 0 then
        FNodes.Delete(I)
      else
        FNodes.Tags[I] := FNodes.Tags[I] - 1
    end;
    Data.RemoveFreeNotification(Self);
  end
end;

procedure TGuard.Notification(AComponent: TComponent; Operation: TOperation);
var
  I: Integer;
  C: Integer;
begin
  if (Operation = opRemove) and (AComponent <> nil) then begin
    I := 0;
    C := FNodes.Count;
    while I < C do begin
      TCustomLinkedStringNode(FNodes[I]).UnLink(AComponent);
      if C = FNodes.Count then
        Inc(I)
      else
        // v dusledku UnLink() se uzel odregistroval
        C := FNodes.Count
    end
  end;
  inherited
end;

destructor TGuard.Destroy;
begin
  inherited;
  FNodes.Free
end;

procedure TLinkedStringNodeManager.Guard(Node: TCustomLinkedStringNode; Data: TComponent; Active: Boolean);
begin
  if not Assigned(FGuard) then
    FGuard := TGuard.Create;
  TGuard(FGuard).Guard(Node, Data, Active)
end;

destructor TLinkedStringNodeManager.Destroy;
begin
  inherited;
  FGuard.Free
end;

end.
