unit uMyskaApp;

interface
uses
  Classes, uMyskaDB;

type
  TMyskaApp = class(uMyskaDB.TMyskaApp)
  protected
    procedure DoInstall; override;
    function DoUpgrade(OldVersion: Integer): Integer; override;
    procedure ConfigureQueries; override;
  protected
    procedure SetID; virtual; abstract;
    function GetScriptText(const ScriptFileName: string): string;
  public
    constructor Create(AnOwner: TComponent); override;
    procedure Connect(AStore: TMyskaStore; const AScriptFolder: string = '');
    function GetScript(const ID: string): string; override;
  end;
  TMyskaAppFactory = class of TMyskaApp;

implementation
uses
  SysUtils, uMyskaMetaTables;

constructor TMyskaApp.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  if Assigned(AnOwner) and (AnOwner is TMyskaStore) then
    Connect(TMyskaStore(AnOwner))
end;

procedure TMyskaApp.Connect(AStore: TMyskaStore; const AScriptFolder: string);
var
  Folder: string;
begin
  Store := AStore;
  SetID;
  if AScriptFolder = '' then
    Folder := ExtractFilePath(ParamStr(0)) + 'dotazy\' + AppID + '\'
  else
    Folder := AScriptFolder;
  if (Folder <> '') and (Folder[Length(Folder)] <> '\') then
    Folder := Folder + '\';
  Params['app.files'] := Folder;
end;

procedure TMyskaApp.DoInstall;
begin
  ExecScript(Subst('{app.files}') + Format('instalace%d.sql', [AppVersion]));
end;

function TMyskaApp.DoUpgrade(OldVersion: Integer): Integer;
begin
  Result := OldVersion + 1;
  ExecScript(Subst('{app.files}') + Format('aktualizace%d.sql', [Result]));
end;

procedure TMyskaApp.ConfigureQueries;
begin
  inherited;
  DatabaseDescriptor(Self)
end;

function TMyskaApp.GetScriptText(const ScriptFileName: string): string;
var
  FileName: string;
  Stream: TStream;
begin
  FileName := Subst('{app.files}') + ScriptFileName;
  Result := '';
  if FileExists(FileName) then begin
    Stream := TFileStream.Create(FileName, fmOpenRead or fmShareDenyWrite);
    try
      SetLength(Result, Stream.Size);
      if Result <> '' then
        Stream.ReadBuffer(Result[1], Length(Result))
    finally
      Stream.Free
    end
  end;
end;

function TMyskaApp.GetScript(const ID: string): string;
begin
  if ID = appScript_XMLDoc then
    Result := GetScriptText('dbdoc.xml')
  else
    Result := inherited GetScript(ID)
end;

end.
