{ VERSION: 1.7 }
{ 1.6 / 1.10.2005
  - opertor 'db.operator.strcat'
  1.5 / 15.5.2005
  - implementuje detektor formtu souboru ValidFile()
  1.4 / 21.4.2005
  - podporuje TMyskaStore.ResetGenID()
  1.3 / 2.4.2005
  - podporuje substituce s ruznym poctem parametru
  1.2 / 9.3.2005
}
unit uMyskaIBStore;

interface
uses
  Classes, uMyskaDB, uMyskaBaseIBStore;

type
  TMyskaIBStore = class(TMyskaBaseIBStore)
  protected
    procedure DoAdd(Sender: TMyskaApp); override;
    function DoPrepareGenerator(const GeneratorName: string): string; override;
    procedure ConfigureQueries; override;
    function ExclusiveAccess: Boolean; override;
  protected
    class function ValidFile(const Header: string): Boolean; override;
  end;

implementation
uses
  SysUtils;

class function TMyskaIBStore.ValidFile(const Header: string): Boolean;
begin
  Result := Copy(Header, 1, 4) = #1#0'90'    // TODO: ?
end;

procedure TMyskaIBStore.DoAdd(Sender: TMyskaApp);
begin
  inherited;
  with Sender.DBQueries do begin
    ArrayValues['app.table.name', 1] := '{app.id}$%s';
    ArrayValues['app.table.name', 2] := '%0:s$%1:s';
    ArrayValues['app.generator.name', 1] := '{app.id}$%s';
  end
end;

procedure TMyskaIBStore.ConfigureQueries;
begin
  inherited;

//  RegisterCommand('db.connection.count', CmdGetConnectedUsers);

  with FDBQueries do begin
    Values['db.operator.upper'] := 'UPPER';
    ArrayValues['db.operator.strcat', 2] := '(COALESCE(%0:s,'''') || COALESCE(%1:s,''''))';
    ArrayValues['db.operator.strcomp', 2] := '(%s=%s)';
    ArrayValues['db.operator.stricomp', 2] := '(UPPER(%s)=UPPER(%s))';
    ArrayValues['db.operator.istrue', 1] := '(%s)=''T''';
    Values['db.const.true'] := '''T''';
    Values['db.const.false'] := '''F''';
    ArrayValues['db.type.string', 1] := 'VARCHAR(%s) CHARACTER SET WIN1250';
    Values['db.type.generator'] := 'INTEGER';
    Values['db.type.date'] := 'DATE';
    Values['db.type.time'] := 'TIME';
    Values['db.type.datetime'] := 'TIMESTAMP';
    Values['db.type.memo'] := 'BLOB SUB_TYPE TEXT CHARACTER SET WIN1250';
    Values['db.type.blob'] := 'BLOB SUB_TYPE 0';
    Values['db.type.double'] := 'DOUBLE PRECISION';
    Values['db.type.money'] := 'DOUBLE PRECISION';
    Values['db.type.integer'] := 'INTEGER';
    ArrayValues['db.type.boolean', 0] := 'CHAR(1) DEFAULT ''F''';
    ArrayValues['db.type.boolean', 1] := 'CHAR(1) DEFAULT %s';

    ArrayValues['db.table.exists', 1] := 'SELECT RDB$RELATION_NAME FROM RDB$RELATIONS WHERE RDB$RELATION_NAME = {db.operator.upper}(''%s'')';
    ArrayValues['db.table.drop', 1] := 'DROP TABLE %s';
    ArrayValues['db.generator.exists', 1] := 'SELECT RDB$GENERATOR_NAME FROM RDB$GENERATORS WHERE RDB$GENERATOR_NAME = {db.operator.upper}(''%s'')';
    ArrayValues['db.generator.create', 1] := 'CREATE GENERATOR %s';
    ArrayValues['db.generator.drop', 1] := 'DROP GENERATOR %s';
    ArrayValues['db.generator.value', 1] := 'SELECT GEN_ID(%s, 0) FROM RDB$DATABASE';
    ArrayValues['db.generator.setValue', 2] := 'SET GENERATOR %0:s TO %1:d'; // 'SELECT GEN_ID(%0:s, %1:d - GEN_ID(%0:s, 0)) FROM RDB$DATABASE';
    ArrayValues['db.generator.increment', 1] := 'SELECT GEN_ID(%s, 1) FROM RDB$DATABASE';

//    Values['db.connection.id'] := 'SELECT CURRENT_CONNECTION FROM RDB$DATABASE';
//    Values['db.connection.count'] := 'db.connection.count()';

    ArrayValues['mys.table.name', 1] := 'MYS$%s';
  end;
end;

const
  MaxGeneratorName = 30;     // TUNE:

function TMyskaIBStore.DoPrepareGenerator(const GeneratorName: string): string;
begin
  Result := UpperCase(StringReplace(GeneratorName, '.', '$', [rfReplaceAll]));
  if Length(Result) > MaxGeneratorName then
    SetLength(Result, MaxGeneratorName);
  if not Exists('{db.generator.exists}', [Result]) then
    Exec('{db.generator.create}', [so_Command], [Result])
end;

function TMyskaIBStore.ExclusiveAccess: Boolean;
var
  Users: TStrings;
begin
  Users := TStringList.Create;
  try
    GetConnectedUsers(Users);
    Result := Users.Count = 1
  finally
    Users.Free
  end
end;

begin
  RegisterFactory(TMyskaIBStore, ['gdb', 'fdb'])
end.
