{ VERSION: 1.3 }
{ 1.2 / 2.10.2005
  - metadata pro aplikaci se nahraj ze TMyskaApp.GetScript(appScript_XMLDoc)
  1.1 / 28.9.2005
}
unit uMyskaMetaTables;

interface
uses
  Classes, DB, uMyskaDB;

const
  appScript_XMLDoc = 'dbdoc';

type
  TFieldDescriptorItem = (
    fdiFieldName,
    fdiOrigin,
    fdiDisplayLabel,
    fdiHint,
    fdiLookupTable,
    fdiGenerator
  );
  TFieldDescriptorItems = set of TFieldDescriptorItem;
  TFieldDescriptor = class
  private
    FDefaultValue: Variant;
    FItems: array [TFieldDescriptorItem] of string;
  protected
    ReferenceField: TFieldDescriptor;
    ReferenceItems: TFieldDescriptorItems;
    function GetString(Index: TFieldDescriptorItem): string;
    procedure SetString(Index: TFieldDescriptorItem; const Value: string);
  public
    property FieldName: string index fdiFieldName read GetString;
    property Origin: string index fdiOrigin read GetString;
    property DisplayLabel: string index fdiDisplayLabel read GetString;
    property Hint: string index fdiHint read GetString;
    property LookupTable: string index fdiLookupTable read GetString;
    property DefaultValue: Variant read FDefaultValue;
    property Generator: string index fdiGenerator read GetString;
  end;

  TFieldDescriptors = class
  private
    FItems: TStrings;
    function GetItem(Index: Integer): TFieldDescriptor;
  protected
    procedure Clear;
    function Add(const FieldName: string): TFieldDescriptor;
  public
    constructor Create;
    destructor Destroy; override;
  public
    function Count: Integer;
    function Find(const FieldName: string): TFieldDescriptor;
    property Items[Index: Integer]: TFieldDescriptor read GetItem; default;
  end;

  TTableDescriptors = class;
  TTableDescriptor = class(TComponent)
  private
    FCollection: TTableDescriptors;
    FTableName: string;
    FHint: string;
    FFields: TFieldDescriptors;
    FKeyField: string;
    FValueField: string;
    FValueExpression: string;
    FLookupCache: TDataSet;
    function GetValueField: string;
    function GetValueExpression: string;
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
  public
    property Collection: TTableDescriptors read FCollection;
    property TableName: string read FTableName;
    property Hint: string read FHint;
    property Fields: TFieldDescriptors read FFields;
    property KeyField: string read FKeyField;
  public
    function LookupCache: TDataSet;
    property ValueField: string read GetValueField;
  end;

  TDatabaseDescriptor = class;
  TTableDescriptors = class
  private
    FCollection: TDatabaseDescriptor;
    FItems: TStrings;
    function GetItem(Index: Integer): TTableDescriptor;
  protected
    procedure Clear;
    function Add(const TableName: string): TTableDescriptor;
  public
    constructor Create;
    destructor Destroy; override;
  public
    property Collection: TDatabaseDescriptor read FCollection;
    function Count: Integer;
    function Find(const TableName: string): TTableDescriptor;
    property Items[Index: Integer]: TTableDescriptor read GetItem; default;
  end;

  TDatabaseDescriptor = class(TComponent)
  private
    FStore: TMyskaStore;
    FTables: TTableDescriptors;
    FIDs: TStrings;
  protected
    procedure Load(const Script: string; const AnID: string; const AVersion: string); overload;
    procedure Load(AnApp: TMyskaApp); overload;
  public
    constructor Create(AnOwner: TComponent); override;
    destructor Destroy; override;
    property Tables: TTableDescriptors read FTables;
    property Store: TMyskaStore read FStore;
  end;

function DatabaseDescriptor(AStore: TMyskaStore): TDatabaseDescriptor; overload;
function DatabaseDescriptor(AnApp: TMyskaApp): TDatabaseDescriptor; overload;

implementation
uses
  SysUtils, XMLNode;

function TFieldDescriptor.GetString(Index: TFieldDescriptorItem): string;
begin
  if Assigned(ReferenceField) and (Index in ReferenceItems) then
    Result := ReferenceField.GetString(Index)
  else
    Result := FItems[Index]
end;

procedure TFieldDescriptor.SetString(Index: TFieldDescriptorItem; const Value: string);
begin
  FItems[Index] := Value
end;

// --------------------------------------------------------------------------

constructor TFieldDescriptors.Create;
begin
  FItems := TStringList.Create;
  TStringList(FItems).Sorted := True;
  inherited Create;
end;

destructor TFieldDescriptors.Destroy;
begin
  inherited;
  Clear;
  FItems.Free;
end;

function TFieldDescriptors.Count: Integer;
begin
  Result := FItems.Count
end;

procedure TFieldDescriptors.Clear;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    Items[I].Free;
  FItems.Clear
end;

function TFieldDescriptors.GetItem(Index: Integer): TFieldDescriptor;
begin
  Result := TFieldDescriptor(FItems.Objects[Index])
end;

function TFieldDescriptors.Add(const FieldName: string): TFieldDescriptor;
var
  Key: string;
begin
  Key := UpperCase(FieldName);
  Result := Find(Key);
  if not Assigned(Result) then begin
    Result := TFieldDescriptor.Create;
    FItems.AddObject(Key, Result);
    Result.SetString(fdiFieldName, Key)
  end
end;

function TFieldDescriptors.Find(const FieldName: string): TFieldDescriptor;
var
  I: Integer;
begin
  I := FItems.IndexOf(FieldName);
  if I < 0 then
    Result := nil
  else
    Result := Items[I]
end;

// ---------------------------------------------------------------------------

constructor TTableDescriptors.Create;
begin
  FItems := TStringList.Create;
  TStringList(FItems).Sorted := True;
  inherited Create;
end;

destructor TTableDescriptors.Destroy;
begin
  inherited;
  Clear;
  FItems.Free;
end;

function TTableDescriptors.Count: Integer;
begin
  Result := FItems.Count
end;

procedure TTableDescriptors.Clear;
var
  I: Integer;
begin
  for I := 0 to Count - 1 do
    Items[I].Free;
  FItems.Clear
end;

function TTableDescriptors.GetItem(Index: Integer): TTableDescriptor;
begin
  Result := TTableDescriptor(FItems.Objects[Index])
end;

function TTableDescriptors.Add(const TableName: string): TTableDescriptor;
var
  Key: string;
begin
  Key := UpperCase(TableName);
  Result := Find(Key);
  if not Assigned(Result) then begin
    Result := TTableDescriptor.Create(nil);
    Result.FCollection := Self;
    FItems.AddObject(Key, Result);
    Result.FTableName := Key
  end
end;

function TTableDescriptors.Find(const TableName: string): TTableDescriptor;
var
  I: Integer;
begin
  I := FItems.IndexOf(TableName);
  if I < 0 then
    Result := nil
  else
    Result := Items[I]
end;

// ---------------------------------------------------------------------------

constructor TTableDescriptor.Create(AnOwner: TComponent);
begin
  inherited Create(AnOwner);
  FFields := TFieldDescriptors.Create
end;

destructor TTableDescriptor.Destroy;
begin
  inherited;
  FFields.Free
end;

function TTableDescriptor.LookupCache: TDataSet;
begin
  Result := FLookupCache;
  if not Assigned(Result) then begin
    FLookupCache := Collection.Collection.Store.Exec('select %s, %s from %s', [so_ReadOnly], [KeyField, GetValueExpression, TableName]);
    Result := FLookupCache
  end
end;

function TTableDescriptor.GetValueExpression: string;
begin
  Result := FValueExpression;
  if Result = '' then begin
    GetValueField;
    Result := FValueExpression
  end
end;

function TTableDescriptor.GetValueField: string;
const
  // TUNE:
  fldValue = 'VALUE_';
  fldSeparator = ';';

  function StrCat(List: TStrings; Index: Integer): string;
  begin
    if Index = List.Count - 1 then
      Result := List[Index]
    else
      Result := Format('{db.operator.strcat(%s,{db.operator.strcat(''' + fldSeparator +''',%s)})}', [List[Index], StrCat(List, Index + 1)])
  end;

var
  I: Integer;
  List: TStrings;
begin
  Result := FValueField;
  if Result = '' then
    for I := 0 to Fields.Count - 1 do
      if not SameText(KeyField, Fields[I].FieldName) then begin
        Result := Fields[I].FieldName;
        FValueField := Result;
        break
      end;
  if Pos(';', Result) = 0 then begin
    FValueExpression := Result
  end else begin
    List := TStringList.Create;
    try
      List.Text := StringReplace(Result, ';', #13#10, [rfReplaceAll]);
      FValueExpression := Format('%s AS %s', [StrCat(List, 0), fldValue]);
    finally
      List.Free
    end;
    FValueField := fldValue;
    Result := FValueField
  end
end;

// ---------------------------------------------------------------------------

var
  DDCachedStore: TMyskaStore = nil;
  DDCachedResult: TDatabaseDescriptor = nil;

function DatabaseDescriptor(AStore: TMyskaStore): TDatabaseDescriptor;
var
  I: Integer;
begin
  Result := nil;
  if Assigned(AStore) then begin
    if AStore = DDCachedStore then
      Result := DDCachedResult
    else begin
      for I := 0 to AStore.ComponentCount - 1 do
        if AStore.Components[I] is TDatabaseDescriptor then begin
          Result := TDatabaseDescriptor(AStore.Components[I]);
          break
        end;
      if not Assigned(Result) then
        Result := TDatabaseDescriptor.Create(AStore);
      DDCachedStore := AStore;
      DDCachedResult := Result;
      Result.FStore := AStore
    end;
  end
end;

function DatabaseDescriptor(AnApp: TMyskaApp): TDatabaseDescriptor;
begin
  Result := DatabaseDescriptor(AnApp.Store);
  Result.Load(AnApp)
end;

// TDatabaseDescriptor -------------------------------------------------------

procedure TDatabaseDescriptor.Load(const Script: string; const AnID: string; const AVersion: string);

  procedure DoLoad(const Script: string);
  const
    xidApp = 'application';
    xidApp_Name = 'id';
    xidApp_Version = 'version';
    xidTable = 'table';
    xidTable_Name = 'id';
    xidTable_Key = 'key';
    xidTable_Value = 'value';
    xidTable_Internal = 'internal';
    xv_Table_Internal = '1';
    xidDescription = 'description';
    xidField = 'field';
    xidField_Name = 'id';
    xidField_LookupTable = 'lookup';
    xidField_DefaultValue = 'default';
    xidField_Generator = 'generator';
    xidTitle = 'title';

    procedure Check(Condition: Boolean);
    begin
      if not Condition then
        raise Exception.Create('')
    end;

    function GetProperty(Node: TXMLNode; const Name: string; const DefaultValue: string = ''): string;
    var
      Attr: TXMLNode;
    begin
      Attr := Node.FindChild(Name, xntAttribute, 0, False);
      if Assigned(Attr) then
        Result := Attr.NodeValue
      else
        Result := DefaultValue
    end;

    function GetElement(Node: TXMLNode; const Name: string = ''): TXMLNode;
    var
      Elem: TXMLNode;
    begin
      Result := nil;
      Elem := Node.FirstChild;
      while Assigned(Elem) do begin
        if Elem.NodeType = xntElement then begin
          if (Name = '') or (Elem.NodeName = Name) then begin
            Result := Elem;
            exit
          end
        end;
        Elem := Elem.NextSibling
      end
    end;

    procedure NextElement(var Node: TXMLNode);
    begin
      Node := Node.NextSibling;
      while Assigned(Node) do
        if Node.NodeType = xntElement then
          break
        else
          Node := Node.NextSibling
    end;

    function NodeText(Node: TXMLNode): string;
    var
      Lines: TStrings;
      I: Integer;
    begin
      Lines := TStringList.Create;
      try
        Lines.Text := Node.Text;
        for I := 0 to Lines.Count - 1 do
          Lines[I] := Trim(Lines[I]);
        Result := TrimRight(Lines.Text)
      finally
        Lines.Free
      end
    end;

    procedure Parse(Node: TXMLNode; Dst: TFieldDescriptor; const OriginPrefix: string; LocalTable: Boolean; const DefaultGenerator: string); overload;
    const
      NoValue = #1;
    var
      S: string;
    begin
      Dst.SetString(fdiLookupTable, GetProperty(Node, xidField_LookupTable));
      Dst.SetString(fdiDisplayLabel, GetProperty(Node, xidField_Name));

      S := GetProperty(Node, xidField_Generator, DefaultGenerator);
      if S <> '' then begin
        if not LocalTable then
          S := '\' + S;
        Dst.SetString(fdiGenerator, S)
      end;

      S := GetProperty(Node, xidField_DefaultValue, NoValue);
      if S <> NoValue then
        Dst.FDefaultValue := S;

      if OriginPrefix <> '' then
        Dst.SetString(fdiOrigin, OriginPrefix + Dst.DisplayLabel);
      Node := GetElement(Node);
      while Assigned(Node) do begin
        if Node.NodeName = xidDescription then begin
          Dst.SetString(fdiHint, NodeText(Node))
        end else
        if Node.NodeName = xidTitle then begin
          Dst.SetString(fdiDisplayLabel, NodeText(Node))
        end else
          Check(False);
        NextElement(Node)
      end;
    end;

    procedure Parse(Node: TXMLNode; Dst: TTableDescriptor); overload;
    var
      TableName: string;
      LocalTable: Boolean;
      FieldName: string;
      DefaultGenerator: string;
    begin
      TableName := GetProperty(Node, xidTable_Name);

      LocalTable := GetProperty(Node, xidTable_Internal) = xv_Table_Internal;

      Dst.FKeyField := GetProperty(Node, xidTable_Key);
      Dst.FValueField := GetProperty(Node, xidTable_Value);
      Node := GetElement(Node);
      while Assigned(Node) do begin
        if Node.NodeName = xidDescription then begin
          // TODO: Parse(Dst.Description, Node)
        end else
        if Node.NodeName = xidTitle then begin
          // TODO: Parse(Dst.Description, Node)
        end else
        if Node.NodeName = xidField then begin
          FieldName := GetProperty(Node, xidField_Name);
          if SameText(FieldName, Dst.KeyField) then
            DefaultGenerator := TableName
          else
            DefaultGenerator := '';
          Parse(Node, Dst.Fields.Add(FieldName), TableName + '.', LocalTable, DefaultGenerator)
        end else
          Check(False);

        NextElement(Node)
      end
    end;

  var
    Doc: TXMLDocument;
    Root: TXMLNode;
    Node: TXMLNode;
  begin
    Doc := TXMLDocument.Create(nil);
    try
      Doc.PreserveWhiteSpace := True;
      Doc.LoadFromString(Script);

      Root := Doc.Document.FindChild(xidApp);
      Check(Assigned(Root));

      Check(
        SameText(GetProperty(Root, xidApp_Name, AnID), AnID) and
        SameText(GetProperty(Root, xidApp_Version, AVersion), AVersion)
      );

      Node := GetElement(Root);
      while Assigned(Node) do begin
        if Node.NodeName = xidTable then begin
          Parse(Node, Tables.Add(GetProperty(Node, xidTable_Name)))
        end else
        if Node.NodeName = xidDescription then begin
          // TODO:
        end else
        if Node.NodeName = xidTitle then begin
          // TODO:
        end else
          Check(False);

        NextElement(Node)
      end;
    finally
      Doc.Free
    end
  end;

begin
  if FIDs.IndexOf(AnID) < 0 then begin
    FIDs.Add(AnID);
    if Script <> '' then
      DoLoad(Script)
  end
end;

procedure TDatabaseDescriptor.Load(AnApp: TMyskaApp);
begin
  if FIDs.IndexOf(AnApp.AppID) < 0 then
    Load(AnApp.GetScript(appScript_XMLDoc), AnApp.AppID, IntToStr(AnApp.AppVersion))
end;

constructor TDatabaseDescriptor.Create(AnOwner: TComponent);
begin
  FTables := TTableDescriptors.Create;
  FTables.FCollection := Self;
  FIDs := TStringList.Create;
  TStringList(FIDs).Sorted := True;
  inherited Create(AnOwner);
end;

destructor TDatabaseDescriptor.Destroy;
begin
  inherited;
  FreeAndNil(FTables);
  FreeAndNil(FIDs);
end;

end.
