unit wEditUsers;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, uMyskaDB, StdCtrls, ComCtrls, Buttons, ExtCtrls, ActnList;

type
  TUserEditor = class(TForm)
    Panel1: TPanel;
    BtnInsert: TSpeedButton;
    BtnEdit: TSpeedButton;
    BtnDelete: TSpeedButton;
    BtnExport: TSpeedButton;
    BtnPassword: TSpeedButton;
    BtnPrint: TSpeedButton;
    View: TListView;
    Actions: TActionList;
    ActionInsert: TAction;
    ActionEdit: TAction;
    ActionDelete: TAction;
    ActionPassword: TAction;
    procedure FormShow(Sender: TObject);
    procedure ViewEdited(Sender: TObject; Item: TListItem; var S: String);
    procedure ViewChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure ActionInsertExecute(Sender: TObject);
    procedure ActionEditExecute(Sender: TObject);
    procedure ActionPasswordExecute(Sender: TObject);
    procedure ActionsUpdate(Action: TBasicAction; var Handled: Boolean);
    procedure ViewResize(Sender: TObject);
  private
    FStore: TMyskaStore;
    procedure LoadUsers;
    function GetSelectedUserID: Cardinal;
    procedure SetSelectedUserID(ID: Cardinal);
    function ItemToID(Item: TListItem): Cardinal;
    function IDToItem(ID: Cardinal): TListItem;
  public
    property Store: TMyskaStore read FStore write FStore;
    property UserID: Cardinal read GetSelectedUserID write SetSelectedUserID;
  end;

function CreateEditor(AnOwner: TComponent; AParent: TWinControl; AStore: TMyskaStore): TUserEditor;

implementation

{$R *.dfm}

function CreateEditor(AnOwner: TComponent; AParent: TWinControl; AStore: TMyskaStore): TUserEditor;
begin
  Result := TUserEditor.Create(AnOwner);
  if Assigned(AParent) then begin
    Result.BorderStyle := bsNone;
    Result.Align := alClient;
    Result.Parent := AParent;
  end;
  Result.Store := AStore
end;

procedure TUserEditor.FormShow(Sender: TObject);
begin
  LoadUsers
end;

function TUserEditor.GetSelectedUserID: Cardinal;
begin
  if View.ItemIndex < 0 then
    Result := 0
  else
    Result := ItemToID(View.Items[View.ItemIndex])
end;

function TUserEditor.ItemToID(Item: TListItem): Cardinal;
begin
  if Assigned(Item) and (Item.SubItems.Count > 0) then
    Result := StrToIntDef(Item.SubItems[0], 0)
  else
    Result := 0
end;

procedure TUserEditor.SetSelectedUserID(ID: Cardinal);
var
  I: Integer;
begin
  for I := 0 to View.Items.Count - 1 do
    if View.Items[I].SubItems[0] = IntToStr(ID) then begin
      View.ItemIndex := I;
      exit
    end;
  View.ItemIndex := -1
end;

function TUserEditor.IDToItem(ID: Cardinal): TListItem;
var
  I: Integer;
begin
  for I := 0 to View.Items.Count - 1 do
    if View.Items[I].SubItems[0] = IntToStr(ID) then begin
      Result := View.Items[I];
      exit
    end;
  Result := nil
end;

procedure TUserEditor.LoadUsers;

  procedure Add(User: TMyskaUser);
  var
    Item: TListItem;
  begin
    Item := View.Items.Add;
    Item.Caption := User.Name;
    Item.SubItems.Add(IntToStr(User.ID));
    Item.Checked := User.Enabled
  end;

var
  I: Integer;
begin
  View.Items.Clear;
  for I := 0 to Store.Users.Count - 1 do
    Add(Store.Users[I]);
  View.OnResize(View)
end;

procedure TUserEditor.ViewEdited(Sender: TObject; Item: TListItem;
  var S: String);
begin
  Store.Users.Find(ItemToID(Item)).Name := S
end;

procedure TUserEditor.ViewChange(Sender: TObject; Item: TListItem;
  Change: TItemChange);
var
  User: TMyskaUser;
begin
  if csDestroying in ComponentState then exit;
  if Change = ctState then begin
    User := Store.Users.Find(ItemToID(Item));
    if Assigned(User) then
      User.Enabled := Item.Checked
  end
end;

procedure TUserEditor.ActionInsertExecute(Sender: TObject);
var
  UserName: string;
  User: TMyskaUser;
begin
  UserName := Format('User%d', [Store.Users.Count + 1]);
  if InputQuery('Nov uivatel', 'Zadej pihlaovac jmno:', UserName) then begin
    User := Store.Users.Add(UserName, '', False);
    LoadUsers;
    UserID := User.ID
  end
end;

procedure TUserEditor.ActionEditExecute(Sender: TObject);
begin
  View.ItemFocused.EditCaption
end;

procedure TUserEditor.ActionPasswordExecute(Sender: TObject);
var
  Password: string;
begin
  Password := '';
  if InputQuery('Zmna hesla', 'Zadej nov heslo:', Password) then
    Store.Users.Find(UserID).Password := Password
end;

procedure TUserEditor.ActionsUpdate(Action: TBasicAction;
  var Handled: Boolean);
var
  Selected: Boolean;
begin
  Selected := View.ItemIndex >= 0;
  ActionEdit.Enabled := Selected;
  ActionPassword.Enabled := Selected and (not Store.Users.WindowsAuthentication);
  Handled := True;
end;

procedure TUserEditor.ViewResize(Sender: TObject);
begin
  with View do Columns[0].Width := ClientWidth - Columns[1].Width
end;

end.
